package chat.client;

import chat.common.*;
import chat.server.*;
import java.rmi.*;
import java.rmi.server.*;


/**
 * Modelo que gerencia a conexo CORBA e os objetos CORBA.
 */
class Client
////////////
{
  private ChatClient chatClient;
  private ChatRoomMgr chatRoomMgr;
  private UserConnection connection;
  
  /**
   * O user atual ou null.
   */
  private String userName;
  
  /**
   * A sala atual ou null.
   */
  private String roomName;


  /**
   * @param clientListener
   *               O callback para o recebimento de mensagens.
   */
  public Client( ChatClient clientListener, String args[] )
    throws Exception
  /////////////////////////////////////////////////////////
  {
    if ( args.length < 1 )
    {
      System.err.println( "** Argument missing: <host>" );  
      System.exit( -1 );
    }

    // -- Install a SecurityManager
    if ( System.getSecurityManager() == null )
    {
      //System.setSecurityManager( new RMISecurityManager() );
    }

    // Cria a instncia servidora.
    ChatClientImpl chatClientImpl = new ChatClientImpl( clientListener );

    final String host = args[ 0 ];
    final String name = "//" + host + "/" + Constants.SERVICE_NAME;

    // --Lookup for remote instance.
    Helper.getInstance().log( "Looking for server at " + host + " under name " + name );
    this.chatRoomMgr = (ChatRoomMgr) Naming.lookup( name );
    Helper.getInstance().log( "Server found." );

    // Activa objeto no ORB.
    this.chatClient = chatClientImpl;
    this.connection = null;

    // --
    this.userName = null;
    this.roomName = null;
    
    Helper.getInstance().log( "Client initialized." );
  }


  void connect( String roomName, String userName )
    throws ChatRoomDoesntExistException, UserAlreadyInRoomException, UserNotInRoomException,
           RemoteException
  /////////////////////////////////////////////////////////////////////////////////////////
  {
    if ( this.connection == null )
    {
      this.connection = this.chatRoomMgr.createNewConnection( roomName, userName, this.chatClient );
      this.roomName = roomName;
      this.userName = userName;
    }
    else
    {
      throw new IllegalStateException( "J conectado. Deve desconetar primeiro." );
    }
  }


  void disconnect()
    throws ChatRoomDoesntExistException, RemoteException
  //////////////////////////////////////////////////////
  {
    if ( this.connection != null )
    {
      // Informa ao servidor.
      this.chatRoomMgr.closeConnection( this.roomName, this.userName );
      this.connection = null;
      this.roomName = null;
      this.userName = null;
    }
  }


  boolean isConnected()
  /////////////////////
  {
    return this.connection != null;
  }


  /**
   * @precondition: deve-se estar numa sala.
   * @return O nome do usurio conectado.
   */
  String getUserName()
  ////////////////////
  {
    return this.userName;
  }


  /**
   * @precondition: deve-se estar numa sala.
   * @return O nome da sala atual.
   */
  String getCurrentRoomName()
  ///////////////////////////
  {
    return this.roomName;
  }



  /**
   * @precondition: deve-se estar numa sala.
   * @return A lista de usurios na sala atual.
   */
  String[] getUserNamesInRoom()
  /////////////////////////////
  {
    String[] names;
    try
    {
      names = this.chatRoomMgr.listUserNames( this.roomName );
    }
    catch ( Exception e )
    {
      names = new String[ 0 ];
      Helper.getInstance().log( e );
    }

    return names;
  }


  /**
   * @precondition: deve-se estar numa sala.
   * @return A lista de usurios na sala atual.
   */
  String[] getRoomNames()
  ///////////////////////
  {
    String[] names;
    try
    {
      names = this.chatRoomMgr.listRoomNames();
    }
    catch ( Exception e )
    {
      names = new String[ 0 ];
      Helper.getInstance().log( e );
    }

    return names;
  }


  /**
   * @precondition: deve-se estar numa sala.
   */
  void sendOperation( GenericOperation gop, String destUserName )
    throws UserNotInRoomException, RemoteException
  {
    this.connection.sendOperation( gop, destUserName );
  }


  /**
   * @precondition: deve-se estar numa sala.
   */
  void broadcastOperation( GenericOperation gop )
    throws RemoteException
  ///////////////////////////////////////////////
  {
    this.connection.broadcastOperation( gop );
  }


  String inviteToHCSession( String sessionName, String destUserName )
    throws UserNotInRoomException, RemoteException
  {
    return this.connection.inviteToHCSession( sessionName, destUserName );
  }


  void testHyperChatSession()
    throws UserNotInRoomException, RemoteException
  {
    // @todo
    this.connection.sendOperation( new DrawOvalOp( this.userName ), "sandoval" );
  }
} // class Client



