package chat.server;

import java.util.HashMap;
import java.util.Iterator;
import java.rmi.RemoteException;
import chat.client.ChatClient;
import chat.client.GenericOperation;
import chat.common.Helper;


/**
 * Modela uma sala de bate papo. Esta classe s  usada no servidor e no  exportada ao cliente.
 */
class ChatRoom
//////////////
{
  /**
   * O nome desta sala.
   */
  private String name;

  /**
   * A descrio desta sala.
   */
  private String description;

  /**
   * As conexes atuais nesta sala (String x UserConnectionImpl).
   *@link aggregationByValue
   *      @associates <{UserConnectionImpl}>
   * @supplierCardinality 0..*
   * @supplierRole userConnections
   */
  private HashMap userConnections;


  public ChatRoom( String name, String description )
  //////////////////////////////////////////////////
  {
    this.name = name;
    this.description = description;
    this.userConnections = new HashMap();
    // Log info.
    Helper.getInstance().log( "ChatRoom instance created: " + name );
  }


  /**
   * Retorna o nome desta sala.
   */
  public String getName()
  ///////////////////////
  {
    return this.name;
  }


  /**
   * Retorna a descrio desta sala.
   */
  public String getDescription()
  //////////////////////////////
  {
    return this.description;
  }


  /**
   * Envia a mensagem a todos os usurios da sala exceto ao origem.
   */
  public void broadcastOperation( GenericOperation gop )
    throws RemoteException
  //////////////////////////////////////////////////////////////////
  {
    for ( Iterator it = this.userConnections.keySet().iterator(); it.hasNext(); )
    {
      String name = ( String )it.next();
      if ( !name.equals( gop.getOperationCreatorName() ) )
      {
        try
        {
          sendOperation( gop, name );
        }
        catch ( UserNotInRoomException e )
        {
          System.err.println( e );
        }
      }
    } // for
  }


  public void sendOperation( GenericOperation gop, String destUserName )
    throws UserNotInRoomException, RemoteException
  {
    // Escolhe o destino e envia a operao para ser executada ali.
    findUserConnectionFor( destUserName ).getChatClient().receiveOperation( gop );
  }


  public String inviteToHCSession( String sessionName, String destUserName )
    throws UserNotInRoomException, RemoteException
  {
    // Escolhe o destino e envia a operao para ser executada ali.
    return findUserConnectionFor( destUserName ).getChatClient().receiveInviteToHCSession( sessionName );
  }


  /**
   * Fecha a conexo para o usurio especificado e notifica aos outros usurios.
   */
  public void closeConnection( String userName )
    throws RemoteException
  //////////////////////////////////////////////
  {
    // Apaga da lista de conexes.
    this.userConnections.remove( userName );
    // Informa aos outros na sala que userName saiu.
    notifyUserLeaved( userName );
    Helper.getInstance().log( "Usuario [" + userName + "] saiu de " + this.name );
  }


  /**
   * Retorna uma nova conexo nesta sala para o usurio especificado. O objeto retornado j est ativado no ORB.
   * @exception UserAlreadyInRoomException Se o usurio j est nesta sala.
   */
  public UserConnection createNewConnection( String userName, ChatClient client ) 
    throws UserAlreadyInRoomException, RemoteException
  ///////////////////////////////////////////////////////////////////////////////
  {
    UserConnectionImpl userConnection = ( UserConnectionImpl )this.userConnections.get( userName );
    if ( userConnection == null )
    {
      // Cria nova conexo.
      userConnection = new UserConnectionImpl( this, userName, client );
      // Registra a conexo.
      this.userConnections.put( userName, userConnection );
      // Informa aos outros na sala que userName entrou (antes de registrar ele mesmo).
      notifyUserEntered( userName );
      Helper.getInstance().log( "Usuario [" + userName + "] entrou em " + this.name );
    }
    else
    {
      throw new UserAlreadyInRoomException( "Ja existe na sala " + this.name + " um usuario com o nome " + userName + "." );
    }
    // Activa objeto no ORB.
    return userConnection;
  }


  /**
   * Retorna os nomes dos usurios nesta sala.
   */
  public String[] listUserNames()
  ///////////////////////////////
  {
    // Faz converso direta com collection-bridge.
    return ( String[] ) this.userConnections.keySet().toArray( new String[ 0 ] );
  }


  /**
   * Verifica se suas conexes esto ok.
   */
  void checkAliveRooms()
  //////////////////////
  {
    // Faz uma copia para evitar modificaes concorrentes.
    java.util.List list = new java.util.ArrayList( this.userConnections.keySet() );
    for ( Iterator it = list.iterator(); it.hasNext(); )
    {
      String name = ( String )it.next();
      try
      {
        findUserConnectionFor( name ).checkAliveStatus();
      }
      catch ( UserNotInRoomException e )
      {
      }
    } // for
  }


  private void notifyUserEntered( String userName )
    throws RemoteException
  /////////////////////////////////////////////////
  {
    for ( Iterator it = this.userConnections.keySet().iterator(); it.hasNext(); )
    {
      String name = ( String )it.next();
      if ( !name.equals( userName ) )
      {
        try
        {
          // Notifica o cliente.
          findUserConnectionFor( name ).getChatClient().userEnteredRoom( userName );
        }
        catch ( UserNotInRoomException e )
        {
          System.err.println( e );
        }
      }
    } // for
  }


  private void notifyUserLeaved( String userName )
    throws RemoteException
  ////////////////////////////////////////////////
  {
    for ( Iterator it = this.userConnections.keySet().iterator(); it.hasNext(); )
    {
      String name = ( String )it.next();
      if ( !name.equals( userName ) )
      {
        try
        {
          // Notifica o cliente.
          findUserConnectionFor( name ).getChatClient().userLeavedRoom( userName );
        }
        catch ( UserNotInRoomException e )
        {
          System.err.println( e );
        }
      }
    } // for
  }


  private UserConnectionImpl findUserConnectionFor( String userName ) throws UserNotInRoomException
  ///////////////////////////////////////////////////////////////////
  {
    UserConnectionImpl conn = ( UserConnectionImpl )this.userConnections.get( userName );
    if ( conn == null )
    {
      throw new UserNotInRoomException( "No existe na sala um usuario com o nome " + userName + "." );
    }
    return conn;
  }
} // class ChatRoom

