package chat.server;

import chat.client.ChatClient;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.Iterator;
import chat.common.Helper;
import java.rmi.*;
import java.rmi.server.*;


/**
 * Gerencia as salas de chat disponveis.
 * Existe uma nica instncia no sistema.
 *
 * @todo Pensar na possibilidade de utilizar o pattern Singleton.
 * @author Jorge F. Del Teglia
 */
class ChatRoomMgrImpl extends UnicastRemoteObject
  implements ChatRoomMgr
{
  /**
   * As chat rooms registradas.
   *
   *@link aggregationByValue
   *      @associates <{ChatRoom}>
   * @supplierRole chatRooms
   * @supplierCardinality 0..*
   */
  private HashMap chatRooms;


  public ChatRoomMgrImpl( String[] roomNames )
    throws RemoteException
  ////////////////////////////////////////////
  {
    this.chatRooms = new HashMap();

    // Log info.
    Helper.getInstance().log( "ChatRoomMgrImpl instance created." );

    // Cria as salas solicitadas.
    createChatRoomsFromNames( roomNames );
    new AliveChecker( this ).start();
  }


  public String createChatRoom( String roomName ) 
    throws ChatRoomAlreadyExistsException, RemoteException
  ///////////////////////////////////////////////
  {
    if ( !this.chatRooms.containsKey( roomName ) )
    {
      // Cria nova ChatRoom.
      // @todo: set some description
      ChatRoom chatRoom = new ChatRoom( roomName, roomName );

      // Registra a sala.
      this.chatRooms.put( roomName, chatRoom );
    }
    else
    {
      throw new ChatRoomAlreadyExistsException( "Ja existe uma sala com o nome " + roomName );
    }
    return roomName;
  }


  public void removeChatRoom( String roomName )
    throws ChatRoomDoesntExistException, RemoteException
  /////////////////////////////////////////////
  {
    if ( this.chatRooms.containsKey( roomName ) )
    {
      this.chatRooms.remove( roomName );
    }
    else
    {
      throw new ChatRoomDoesntExistException( "Nao existe sala com o nome " + roomName + " para ser removida." );
    }
  }


  /**
   * Retorna uma nova conexo com a sala especificada para
   * o usurio especificado.
   *
   * O objeto retornado j est ativado no ORB.
   *
   * @exception ChatRoomDoesntExistException
   *                   Se no existe a sala especificada.
   * @exception UserAlreadyInRoomException
   *                   Se o usurio j est nesta sala.
   */
  public UserConnection createNewConnection( String roomName, String userName, ChatClient client )
    throws ChatRoomDoesntExistException, UserAlreadyInRoomException, RemoteException
  /////////////////////////////////////////////////////////////////////////////
  {
    ChatRoom chatRoom = ( ChatRoom )this.chatRooms.get( roomName );
    if ( chatRoom == null )
    {
      throw new ChatRoomDoesntExistException( "Nao existe sala com o nome " + roomName + "." );
    }
    return chatRoom.createNewConnection( userName, client );
  }


  /**
   * Fecha a conexo indicada.
   */
  public void closeConnection( String roomName, String userName )
    throws ChatRoomDoesntExistException, RemoteException
  ///////////////////////////////////////////////////////////////
  {
    ChatRoom chatRoom = ( ChatRoom )this.chatRooms.get( roomName );
    if ( chatRoom == null )
    {
      throw new ChatRoomDoesntExistException( "Nao existe sala com o nome " + roomName + "." );
    }
    chatRoom.closeConnection( userName );
  }


  public String[] listUserNames( String roomName ) 
    throws ChatRoomDoesntExistException, RemoteException
  ////////////////////////////////////////////////
  {
    ChatRoom chatRoom = ( ChatRoom )this.chatRooms.get( roomName );
    if ( chatRoom == null )
    {
      throw new ChatRoomDoesntExistException( "Nao existe sala com o nome " + roomName + "." );
    }
    return chatRoom.listUserNames();
  }


  /**
   * @return Os nomes das salas existentes.
   */
  public String[] listRoomNames()
  ///////////////////////////////
  {
    // Faz converso direta com collection-bridge.
    return ( String[] ) this.chatRooms.keySet().toArray( new String[ 0 ] );
  }


  /**
   * Solicita a todas as salas para verificar se suas conexes esto ok.
   */
  void checkAliveRooms()
  //////////////////////
  {
    for ( Iterator it = this.chatRooms.entrySet().iterator(); it.hasNext(); )
    {
      Entry entry = ( Entry )it.next();
      ( ( ChatRoom )entry.getValue() ).checkAliveRooms();
    } // for
  }


  /**
   * Cria as salas com os nomes pedidos.
   */
  private void createChatRoomsFromNames( String[] roomNames )
    throws RemoteException
  ///////////////////////////////////////////////////////////
  {
    // Cria as salas solicitadas.
    if ( roomNames != null )
    {
      try
      {
        // Cria chat rooms.
        for ( int i = 0; i < roomNames.length; i++ )
        {
          createChatRoom( roomNames[ i ] );
        }
      }
      catch ( ChatRoomAlreadyExistsException e )
      {
        System.err.println( e.getMessage() );
      }
    }
  }
} // class ChatRoomMgrImpl


/**
 * Gerencia o check para conferir se as conexes esto ativas.
 */
class AliveChecker extends Thread
/////////////////////////////////
{
  /**
   * O numero de segondos entre cada verificao das conexes vivas.
   * Est setado para 10 segondos s a modo de teste e para facilitar a correo.
   */
  private static final int SECONDS_BETWEEN_CHECKS = 10;

  /**
   * @supplierRole owner
   * @supplierCardinality 1
   */
  private ChatRoomMgrImpl owner;


  AliveChecker( ChatRoomMgrImpl owner )
  /////////////////////////////////////
  {
    this.owner = owner;
  }


  public void run()
  /////////////////
  {
    while ( true )
    {
      try
      {
        Thread.sleep( SECONDS_BETWEEN_CHECKS * 1000 );
      }
      catch ( InterruptedException e )
      {
      }
      this.owner.checkAliveRooms();
    } // while
  }
}


