package chat.server;

import chat.client.*;
import chat.common.Helper;
import java.rmi.*;
import java.rmi.server.*;

/**
 * Gerencia a conexo de um usurio com uma sala de chat.
 * @author Jorge F. Del Teglia
 */
class UserConnectionImpl extends UnicastRemoteObject
  implements UserConnection
{
  /**
   * O nome do usurio usando esta conexo.
   */
  private String userName;

  /**
   * O momento em que esta conexo foi criada (em milliseconds.)
   */
  private long connectionStartTime;

  /**
   * A chat room onde est conectado.
   * @supplierRole chatRoom
   * @supplierCardinality 1
   */
  private ChatRoom chatRoom;

  private ChatClient client;

  /**
   * Inicializa uma conexo especificando em que sala e com que usurio.
   */
  public UserConnectionImpl( ChatRoom chatRoom, String userName, ChatClient client )
    throws RemoteException
  //////////////////////////////////////////////////////////////////////////////////
  {
    this.chatRoom = chatRoom;
    this.userName = userName;
    this.client   = client;
    this.connectionStartTime = System.currentTimeMillis();

    // Log info.
    Helper.getInstance().log( "UserConnectionImpl instance created: " + userName );
  }


  /**
   * @return o nome do usurio conectado.
   */
  public String getUserName()
  ///////////////////////////
  {
    return this.userName;
  }


  /**
   * @return o nome da sala onde esta connexo est.
   */
  public String getChatRoomName()
  ///////////////////////////////
  {
    return this.chatRoom.getName();
  }


  /**
   * @return o numero de ms desde que a conexo foi estabelecida.
   */
  public long getConnectionTime()
  ///////////////////////////////
  {
    return (System.currentTimeMillis() - this.connectionStartTime);
  }


  /**
   * Envia a mensagem a todos os usurios da sala exceto ele mesmo.
   */
  public void broadcastOperation( GenericOperation gop )
    throws RemoteException
  //////////////////////////////////////////////////////
  {
    // Delega ao chat room.
    this.chatRoom.broadcastOperation( gop );
  }


  public void sendOperation( GenericOperation gop, String destUserName )
    throws UserNotInRoomException, RemoteException
  {
    // Delega ao chat room.
    this.chatRoom.sendOperation( gop, destUserName );
  }
  
  
  public String inviteToHCSession( String sessionName, String destUserName )
    throws UserNotInRoomException, RemoteException
  {
    return this.chatRoom.inviteToHCSession( sessionName, destUserName );
  }

  
  ChatClient getChatClient()
  //////////////////////////
  {
    return this.client;
  }


  void checkAliveStatus()
  ///////////////////////
  {
    String msg = "Verificando conexao " + this.userName + " em " +
                 this.chatRoom.getName() + ": ";

    try
    {
      // Chama a um mtodo que no faz nada para verificar a conexo.
      this.client.checkAlive();
      Helper.getInstance().log( msg + "Ok" );
    }
    catch ( Exception e )
    {
      // No consiguiou conetar.
      Helper.getInstance().log( msg + "Error - fechando conexao" );
      try
      {
        this.chatRoom.closeConnection( this.userName );
      }
      catch ( Exception e1 )
      {
        e.printStackTrace();
      }
    }
  }

} // class UserConnectionImpl
